
/*
 * File Monotonicity.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#ifndef __Monotonicity__
#define __Monotonicity__

#include "Forwards.hpp"

#include "Lib/Stack.hpp"
#include "Lib/DHMap.hpp"
#include "Lib/ScopedPtr.hpp"

#include "Kernel/Term.hpp"
#include "Kernel/Clause.hpp"
#include "Kernel/SortHelper.hpp"

#include "SAT/SATSolver.hpp"
#include "SAT/SATLiteral.hpp"
#include "SAT/SATClause.hpp"

#include "Lib/Allocator.hpp"

namespace FMB {
  using namespace Kernel;
  using namespace SAT;
  using namespace Lib;

class Monotonicity{

  CLASS_NAME(Monotonicity);
  USE_ALLOCATOR(Monotonicity);

public:
  
  Monotonicity(ClauseList* clauses, unsigned srt);

  bool check(){ return _result;}

  static void addSortPredicates(bool withMon, ClauseList*& clauses, DArray<unsigned>& del_f);
  static void addSortFunctions(bool withMon, ClauseList*& clauses);

private:
  
  void monotone(Clause* c, Literal* l); 
  
  void safe(Clause* c, Literal* l, TermList* t); 
  void safe(Clause* c, Literal* l, TermList* t, SATLiteral add); 
  void safe(Clause* c, Literal* l, TermList* t, Stack<SATLiteral>& slits);

  
  
  bool guards(Literal* l, unsigned var, Stack<SATLiteral>& slits);

 
  unsigned _srt; 
  
  
  bool _result;

  DHMap<unsigned,SATLiteral> _pF;
  DHMap<unsigned,SATLiteral> _pT;

  ScopedPtr<SATSolver> _solver;

};

}
#endif
