
/*
 * File FormulaIndex.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#ifndef __FormulaIndex__
#define __FormulaIndex__

#include "Forwards.hpp"

#include "Lib/MapToLIFO.hpp"


#include "Index.hpp"

namespace Indexing {

using namespace Lib;
using namespace Kernel;

class FormulaIndex {
public:
  virtual ~FormulaIndex() {}

  virtual FormulaQueryResultIterator getEquivalent(Formula* f) = 0;

  virtual void insert(FormulaUnit* unit, Formula* f) = 0;
  virtual void remove(FormulaUnit* unit, Formula* f) = 0;
};

class StringFormulaIndex : public FormulaIndex {
public:
  virtual FormulaQueryResultIterator getEquivalent(Formula* f);

  virtual void insert(FormulaUnit* unit, Formula* f);
  virtual void remove(FormulaUnit* unit, Formula* f);
private:
  vstring getKey(Formula* f);

  struct Entry
  {
    Entry(FormulaUnit* unit, Formula* formula) : unit(unit), formula(formula) {}

    bool operator==(const Entry& o) const { return unit==o.unit && formula==o.formula; }
    bool operator!=(const Entry& o) const { return !((*this)==o); }

    FormulaUnit* unit;
    Formula* formula;
  };

  struct Entry2QR;

  MapToLIFO<vstring,Entry> _map;
};


}

#endif 
