
/*
 * File BinaryResolution.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#ifndef __BinaryResolution__
#define __BinaryResolution__

#include "Forwards.hpp"

#include "InferenceEngine.hpp"
#include "Kernel/Ordering.hpp"

namespace Inferences
{

using namespace Kernel;
using namespace Indexing;
using namespace Saturation;

class BinaryResolution
: public GeneratingInferenceEngine
{
public:
  CLASS_NAME(BinaryResolution);
  USE_ALLOCATOR(BinaryResolution);

  BinaryResolution() : _index(0), _unificationWithAbstraction(false) {}

  void attach(SaturationAlgorithm* salg);
  void detach();

  static Clause* generateClause(Clause* queryCl, Literal* queryLit, SLQueryResult res, const Options& opts, Limits* limits=0, Ordering* ord=0, LiteralSelector* ls = 0);
  ClauseIterator generateClauses(Clause* premise);

private:
  struct UnificationsFn;
  struct ResultFn;

  GeneratingLiteralIndex* _index;
  bool _unificationWithAbstraction;
};

};

#endif
