
/*
 * File EquationalTautologyRemoval.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#include "EquationalTautologyRemoval.hpp"

#include "Lib/DHSet.hpp"
#include "Lib/Environment.hpp"
#include "Shell/Statistics.hpp"
#include "Kernel/Clause.hpp"

namespace Inferences
{

Clause* EquationalTautologyRemoval::simplify(Clause* cl)
{
  CALL("EquationalTautologyRemoval::simplify");

  
  
  
  

  bool has_neg_eq = false;
  bool has_pos_eq = false;
  bool has_complement = false;

  static DHSet<int> preds;
  preds.reset();

  for (unsigned i = 0; i < cl->length(); i++) {
    Literal* lit = (*cl)[i];

    if (lit->isEquality()) {
      if (lit->isNegative()) {
        has_neg_eq = true;
      } else {
        has_pos_eq = true;
      }
    } else {
      ASS_G(lit->functor(),0);

      int pred = lit->isNegative() ? -lit->functor() : lit->functor();

      if (preds.find(-pred)) {
        has_complement = true;
      }

      preds.insert(pred);
    }
  }

  if (!has_neg_eq || (!has_pos_eq && !has_complement)) {
    return cl;
  }

  _cc.reset();

  
  for (unsigned i = 0; i < cl->length(); i++) {
    Literal* lit = (*cl)[i];

    if (!lit->isEquality()) {
      int pred = lit->isNegative() ? -lit->functor() : lit->functor();
      if (!preds.find(-pred)) {
        continue;
      }
    }

    
    Literal* oplit = Literal::complementaryLiteral(lit);
    _cc.addLiteral(oplit);
  }

  if (_cc.getStatus(false) == DP::DecisionProcedure::UNSATISFIABLE) {
    

    env.statistics->deepEquationalTautologies++;
    return 0;
  } else {
    return cl;
  }
}

}
