
/*
 * File ClauseQueue.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#ifndef __ClauseQueue__
#define __ClauseQueue__

#if VDEBUG
#include <ostream>
using namespace std;
#endif

#include "Debug/Assertion.hpp"

#include "Lib/Reflection.hpp"

namespace Kernel {

class Clause;

class ClauseQueue
{
public:
  ClauseQueue();
  virtual ~ClauseQueue();
  void insert(Clause*);
  bool remove(Clause*);
  void removeAll();
  Clause* pop();
 
  bool isEmpty() const
  { return _left->nodes[0] == 0; }
#if VDEBUG
  void output(ostream&) const;
#endif

  friend class Iterator;
protected:
 
  virtual bool lessThan(Clause*,Clause*) = 0;
 
  class Node {
  public:
   
    Clause* clause;
   
    Node* nodes[1];
  };
 
  unsigned _height;
 
  Node* _left;

public:
 
  class Iterator {
  public:
    DECL_ELEMENT_TYPE(Clause*);

   
    inline explicit Iterator(ClauseQueue& queue)
      : _current(queue._left)
    {}
   
    inline bool hasNext() const
    { return _current->nodes[0]; }
   
    inline Clause* next()
    {
      _current = _current->nodes[0];
      ASS(_current);
      return _current->clause;
    }
  private:
   
    Node* _current;
  }; 
















}; 

} 

#endif
