
/*
 * File ELiteralSelector.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */



#ifndef __ELiteralSelector__
#define __ELiteralSelector__

#include "Forwards.hpp"
#include "Lib/SmartPtr.hpp"
#include "Ordering.hpp"

#include "LiteralSelector.hpp"

namespace Kernel {

class ELiteralSelector
: public LiteralSelector
{
public:
  CLASS_NAME(ELiteralSelector);
  USE_ALLOCATOR(ELiteralSelector);

  enum Values {
    
    SelectNegativeLiterals = 0,
    SelectPureVarNegLiterals = 1,
    
    
    SelectSmallestNegLit = 2,
    
    SelectDiffNegLit = 3,
    SelectGroundNegLit = 4,
    SelectOptimalLit = 5
  };

  ELiteralSelector(const Ordering& ordering, const Options& options, Values value) :
    LiteralSelector(ordering, options), _value(value) {}

  bool isBGComplete() const override { return true; }
protected:
  void doSelection(Clause* c, unsigned eligible);

private:
  LiteralList* getMaximalsInOrder(Clause* c, unsigned eligible);

  unsigned lit_standard_diff(Literal* lit);
  unsigned lit_sel_diff_weight(Literal* lit);

  Values _value;
};

};

#endif
