
/*
 * File FormulaVarIterator.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#ifndef __FormulaVarIterator__
#define __FormulaVarIterator__

#include "Lib/MultiCounter.hpp"
#include "Lib/Stack.hpp"

#include "Kernel/Term.hpp"
#include "Kernel/Formula.hpp"

using namespace Lib;

namespace Kernel {

class FormulaVarIterator
{
public:
  explicit FormulaVarIterator(const Formula*);
  explicit FormulaVarIterator(const Term*);
  explicit FormulaVarIterator(const TermList*);

  bool hasNext();
  int next();

private:
 
  enum Instruction {
   
    FVI_FORMULA,
   
    FVI_TERM,
   
    FVI_TERM_LIST,
   
    FVI_BIND,
   
    FVI_UNBIND,
  };

 
  bool _found;
 
  unsigned _nextVar;

 
  MultiCounter _bound;
 
  MultiCounter _free;
 
  Stack<const Formula*> _formulas;
 
  Stack<const Term*> _terms;
 
  Stack<TermList> _termLists;
 
  Stack<Instruction> _instructions;
 
  Stack<const Formula::VarList*> _vars;
}; 

}

#endif 
