
/*
 * File FallbackSolverWrapper.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#include "Lib/Environment.hpp"
#include "Shell/Statistics.hpp"

#include "Debug/RuntimeStatistics.hpp"

#include "SAT/SATClause.hpp"

#include "FallbackSolverWrapper.hpp"

namespace SAT
{

FallbackSolverWrapper::FallbackSolverWrapper(SATSolver* inner,SATSolver* fallback)
 : _inner(inner), _fallback(fallback), _usingFallback(false),  _varCnt(0) 
{
  CALL("FallbackSolverWrapper::FallbackSolverWrapper");
}

void FallbackSolverWrapper::addClause(SATClause* cl)
{
  CALL("FallbackSolverWrapper::addClause");
  
  _inner->addClause(cl);
  _fallback->addClause(cl);
}

SATSolver::Status FallbackSolverWrapper::solve(unsigned conflictCountLimit)
{
  CALL("FallbackSolverWrapper::solve"); 
  
  
  Status status = _inner->solve(conflictCountLimit);

  
  if(status == Status::UNKNOWN){
    status = _fallback->solve(conflictCountLimit);
    _usingFallback = true;
    ASS(status != Status::UNKNOWN);
    env.statistics->smtFallbacks++;
  } 
  else{
    _usingFallback = false;
  }
  return status;
}

SATSolver::VarAssignment FallbackSolverWrapper::getAssignment(unsigned var)
{
  CALL("FallbackSolverWrapper::getAssignment");
  ASS_G(var,0); ASS_LE(var,_varCnt);

  if(_usingFallback){
    return _fallback->getAssignment(var);
  }
  return _inner->getAssignment(var); 
}

}
