
/*
 * File ProofSimplifier.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#ifndef __ProofSimplifier__
#define __ProofSimplifier__

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"
#include "Lib/Stack.hpp"

#include "Kernel/InferenceStore.hpp"
#include "Kernel/Problem.hpp"

#include "AIG.hpp"
#include "AIGInliner.hpp"
#include "AIGCompressor.hpp"

namespace Shell {

using namespace Lib;
using namespace Kernel;

class ProofTransformer {
public:
  ProofTransformer(Unit* refutation);
  virtual ~ProofTransformer() {}

  void perform();

  Unit* getNewRefutation() {
    CALL("ProofTransformer::getNewRefutation");
    ASS(isRefutation(_newProof.top()));
    return _newProof.top();
  }
protected:
  virtual void preTransform() {}
  virtual Unit* transformUnit(Unit* u) = 0;

  static bool isRefutation(Unit* u);
  static void loadProof(Unit* refutation, Stack<Unit*>& tgt);

  Stack<Unit*> _origProof;
private:

  void derefInference(Unit* src, Unit* tgt);
  void registerTransformation(Unit* src, Unit* tgt);

  Unit* _refutation;

  DHMap<Unit*,Unit*> _transformationMap;

  Stack<Unit*> _newProof;
};


class ProofSimplifier : public ProofTransformer {
public:
  ProofSimplifier(const Problem& prb, Unit* refutation, UnitList* defs);
protected:
  virtual void preTransform();
  virtual Unit* transformUnit(Unit* u);

private:
  AIGRef getAIG(Unit* u);

  
  UnitList* _defs;

  AIGInliner _inl;
  AIG& _aig;
  AIGFormulaSharer& _fsh;
  BDDAIG _bddAig;
};


}

#endif 
