
/*
 * File Skolem.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#ifndef __Skolem__
#define __Skolem__

#include "Lib/DHMap.hpp"
#include "Lib/Stack.hpp"

#include "Kernel/Formula.hpp"
#include "Kernel/Substitution.hpp"

namespace Kernel {
  class Unit;
}

using namespace Lib;
using namespace Kernel;

namespace Shell {




class Skolem
{
public:
  static FormulaUnit* skolemise(FormulaUnit*);
  static unsigned addSkolemFunction(unsigned arity, unsigned* domainSorts,
      unsigned rangeSort, unsigned var);
  static unsigned addSkolemFunction(unsigned arity, unsigned* domainSorts,
      unsigned rangeSort, const char* suffix=0);
  static unsigned addSkolemPredicate(unsigned arity, unsigned* domainSorts, unsigned var);
  static unsigned addSkolemPredicate(unsigned arity, unsigned* domainSorts, const char* suffix=0);
private:
 
  Skolem () :  _beingSkolemised(0) {}
  FormulaUnit* skolemiseImpl(FormulaUnit*);

  
  void preskolemise(Formula*);
  
  Formula* skolemise(Formula*);
  FormulaList* skolemise(FormulaList*);

  void ensureHavingVarSorts();

 
  Substitution _subst;

  typedef List<bool> BoolList;

 
  struct VarOccInfo {
    bool existential;
    BoolList* occurs_below;
  };
  typedef DHMap<unsigned,VarOccInfo> VarOccInfos;
  VarOccInfos _varOccs;

  struct ExVarDepInfo {
    VarSet* univ;
    VarSet* exist;
  };

  typedef DHMap<Formula*,ExVarDepInfo> ExVarDepInfos; 
  ExVarDepInfos _varDeps;

  
  DHMap<unsigned, Formula*> _blockLookup;

 
  DHMap<unsigned,unsigned> _varSorts;

  Stack<unsigned> _introducedSkolemFuns;

  FormulaUnit* _beingSkolemised;

  UnitList* _skolimizingDefinitions;

}; 

}

#endif
