
/*
 * File SymbolOccurrenceReplacement.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */
#ifndef __SymbolOccurrenceReplacement__
#define __SymbolOccurrenceReplacement__

#include "Forwards.hpp"

#include "Kernel/Signature.hpp"
#include "Lib/Environment.hpp"

using namespace Lib;
using namespace Kernel;
using namespace Shell;


class SymbolOccurrenceReplacement {
  public:
   
    SymbolOccurrenceReplacement(bool isPredicate, unsigned symbol, unsigned freshSymbol, Formula::VarList* freeVars)
            : _isPredicate(isPredicate), _symbol(symbol), _freshSymbol(freshSymbol), _freeVars(freeVars) {
        
        ASS_NEQ(symbol, freshSymbol);
        
        
        ASS(freeVars || !env.signature->getFunction(symbol)->introduced());
        
        if (isPredicate) {
            ASS_EQ(env.signature->getPredicate(symbol)->arity() + freeVars->length(),
                   env.signature->getPredicate(freshSymbol)->arity());
        } else {
            ASS_EQ(env.signature->getFunction(symbol)->arity() + freeVars->length(),
                   env.signature->getFunction(freshSymbol)->arity());
        }
    }
    Formula* process(Formula* formula);
    FormulaList* process(FormulaList* formulas);
    Term* process(Term* term);
    TermList process(TermList ts);

  private:
    const bool _isPredicate;
    const unsigned _symbol;
    const unsigned _freshSymbol;
    const Formula::VarList* _freeVars;
};

#endif 