
/*
 * File TheoryAxioms.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */


#ifndef __TheoryAxioms__
#define __TheoryAxioms__

#include "Forwards.hpp"

#include "Kernel/Theory.hpp"
#include "Options.hpp"

namespace Shell {

using namespace Lib;
using namespace Kernel;

class TheoryAxioms {
public:
  TheoryAxioms() {} 

static unsigned const CHEAP = 0;
static unsigned const EXPENSIVE = 1;

  void apply(Problem& prb);
  bool apply(UnitList*& units, Property* prop);

 
  void applyFOOL(Problem& prb);

private:

  void addCommutativity(Interpretation op, UnitList*& units);
  void addAssociativity(Interpretation op, UnitList*& units);
  void addRightIdentity(Interpretation op, TermList idElement, UnitList*& units);
  void addLeftIdentity(Interpretation op, TermList idElement, UnitList*& units);
  void addCommutativeGroupAxioms(Interpretation op,Interpretation inverse,
				 TermList idElement, UnitList*& units);

  void addRightInverse(Interpretation op, Interpretation inverse, UnitList*& units);

  void addNonReflexivity(Interpretation op, UnitList*& units);
  void addTransitivity(Interpretation op, UnitList*& units);
  void addOrderingTotality(Interpretation less, UnitList*& units);
  void addTotalOrderAxioms(Interpretation less, UnitList*& units);
  void addMonotonicity(Interpretation less, Interpretation addition, UnitList*& units);
  void addPlusOneGreater(Interpretation plus, TermList oneElement,
                                     Interpretation less, UnitList*& units);
  void addAdditionAndOrderingAxioms(Interpretation plus, Interpretation unaryMinus,
				    TermList zeroElement, TermList oneElement,
				    Interpretation less, UnitList*& units);
  void addAdditionOrderingAndMultiplicationAxioms(Interpretation plus, Interpretation unaryMinus,
						  TermList zeroElement, TermList oneElement,
						  Interpretation less, Interpretation multiply,
						  UnitList*& units);
  void addExtraIntegerOrderingAxiom(Interpretation plus, TermList oneElement, Interpretation less,
				    UnitList*& units);
  void addQuotientAxioms(Interpretation quotient, Interpretation multiply, TermList zeroElement, TermList oneElement,
                         Interpretation less,UnitList*& units);
  void addIntegerDivisionWithModuloAxioms(Interpretation plus, Interpretation unaryMinus, Interpretation less,
                                Interpretation multiply, Interpretation divide, Interpretation divides,
                                Interpretation modulo, Interpretation abs, TermList zeroElement,
                                TermList oneElement, UnitList*& units);
  void addIntegerAbsAxioms(Interpretation abs, Interpretation less,
                           Interpretation unaryMinus, TermList zeroElement, UnitList*& units);
  void addIntegerDividesAxioms(Interpretation divides, Interpretation multiply, TermList zero, TermList n, UnitList*& units);

  void addBooleanArrayExtensionalityAxioms(Interpretation select, Interpretation store, unsigned skolem, UnitList*& units);
  void addArrayExtensionalityAxioms(Interpretation select, Interpretation store, unsigned skolem, UnitList*& units);
  void addBooleanArrayWriteAxioms(Interpretation select, Interpretation store, UnitList*& units);
  void addTupleAxioms(unsigned tupleSort, UnitList*& units);
  void addFloorAxioms(Interpretation floor, Interpretation less, Interpretation unaryMinus,
                      Interpretation plus, TermList oneElement, UnitList*& units);
  void addCeilingAxioms(Interpretation ceiling, Interpretation less, Interpretation plus, 
                        TermList oneElement, UnitList*& units);
  void addRoundAxioms(Interpretation round, Interpretation floor, Interpretation ceiling, UnitList*& units); 
  void addTruncateAxioms(Interpretation truncate, Interpretation less, Interpretation unaryMinus,
                      Interpretation plus, TermList zeroElement, TermList oneElement, UnitList*& units);
  void addArrayWriteAxioms(Interpretation select, Interpretation store, UnitList*& units);

  
  void addExhaustivenessAxiom(TermAlgebra* ta, UnitList*& units);
  void addDistinctnessAxiom(TermAlgebra* ta, UnitList*& units);
  void addInjectivityAxiom(TermAlgebra* ta, UnitList*& units);
  void addAlternativeInjectivityAxiom(TermAlgebra* ta, UnitList*& units);
  void addDiscriminationAxiom(TermAlgebra* ta, UnitList*& units);
  void addAcyclicityAxiom(TermAlgebra* ta, UnitList*& units);

 
  static bool addSubtermDefinitions(unsigned subtermPredicate, TermAlgebraConstructor* c, UnitList*& units);

  static void addTheoryUnitClause(Literal* lit, UnitList*& units,unsigned level);
  static void addTheoryUnitClause(Literal* lit, Inference* inf, UnitList*& units,unsigned level);
  static void addTheoryNonUnitClause(UnitList*& units, Literal* lit1, Literal* lit2,unsigned level);
  static void addTheoryNonUnitClause(UnitList*& units, Literal* lit1, Literal* lit2, Literal* lit3,unsigned level);
  static void addTheoryNonUnitClause(UnitList*& units, Literal* lit1, Literal* lit2, Literal* lit3, Literal* lit4,unsigned level);
  static void addAndOutputTheoryUnit(Unit* unit,UnitList*& units,unsigned level);
};

}

#endif 
