
/*
 * File TheoryFlattening.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-26. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-26 only because so is required by the CASC-26
 * rules, and for no other reason.
 */



#ifndef __TheoryFlattening__
#define __TheoryFlattening__

#include "Forwards.hpp"
#include "Lib/DHMap.hpp"
#include "Kernel/Term.hpp"

namespace Shell {

using namespace Kernel;

class TheoryFlattening
{
public:
  TheoryFlattening() : _recursive(true), _sharing(false) {}
  TheoryFlattening(bool rec, bool share); 
  void apply(Problem& prb);
  bool apply(UnitList*& units);
  bool apply(ClauseList*& units);
  Clause* apply(Clause*& cl,Stack<Literal*>& target);
  Clause* apply(Clause*& cl){
    static Stack<Literal*> dummy;
    return apply(cl,dummy);
  }
private:
  Literal* replaceTopTerms(Literal* lit, Stack<Literal*>& newLits,unsigned& maxVar,
                           DHMap<Term*,unsigned>& abstracted);
  Term* replaceTopTermsInTerm(Term* term, Stack<Literal*>& newLits,
                              unsigned& maxVar,bool interpreted,
                              DHMap<Term*,unsigned>& abstracted);

  bool _recursive;
  bool _sharing;
};

};

#endif
